package locales

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestLocale_GetLanguage(t *testing.T) {
	tests := []struct {
		code     string
		wantLang string
	}{
		{
			code:     "ru",
			wantLang: "ru",
		},
		{
			code:     "ru_RU",
			wantLang: "ru",
		},
		{
			code:     "en-US",
			wantLang: "en",
		},
		{
			code:     "incorrect-code",
			wantLang: "",
		},
	}

	for _, tt := range tests {
		t.Run(tt.code, func(t *testing.T) {
			locale := &Locale{Code: tt.code}
			require.Equal(t, tt.wantLang, locale.GetLanguage())
		})
	}
}

func TestLocale_Equal(t *testing.T) {
	tests := []struct {
		locale *Locale
		other  *Locale
		want   bool
	}{
		{
			locale: &Locale{Code: "ru-RU"},
			other:  &Locale{Code: "ru-RU"},
			want:   true,
		},
		{
			locale: &Locale{Code: "ru-RU"},
			other:  &Locale{Code: "ru-RU", Fallback: "en-US"},
			want:   false,
		},
		{
			locale: &Locale{Code: "ru-RU", Fallback: "en-US"},
			other:  &Locale{Code: "ru-RU"},
			want:   false,
		},
		{
			locale: &Locale{Code: "ru-RU"},
			other:  nil,
			want:   false,
		},
		{
			locale: nil,
			other:  &Locale{Code: "ru-RU"},
			want:   false,
		},
		{
			locale: nil,
			other:  nil,
			want:   true,
		},
	}
	for _, tt := range tests {
		got := tt.locale.Equal(tt.other)
		require.Equal(t, tt.want, got)
	}
}

func TestFallbackSort(t *testing.T) {
	tests := []struct {
		name    string
		input   []*Locale
		want    []*Locale
		wantErr bool
	}{
		{
			name:    "Empty input",
			input:   nil,
			want:    nil,
			wantErr: false,
		},
		{
			name:    "Empty input",
			input:   []*Locale{},
			want:    []*Locale{},
			wantErr: false,
		},
		{
			name: "One locale not default",
			input: []*Locale{
				{ID: "a"},
			},
			want: []*Locale{
				{ID: "a"},
			},
			wantErr: false,
		},
		{
			name: "One default locale",
			input: []*Locale{
				{ID: DefaultID},
			},
			want: []*Locale{
				{ID: DefaultID},
			},
			wantErr: false,
		},
		{
			name: "One locale with fallback",
			input: []*Locale{
				{ID: DefaultID, Fallback: "a"},
			},
			wantErr: true,
		},
		{
			name: "Fallback locale not exists",
			input: []*Locale{
				{ID: "a", Fallback: "c"},
				{ID: "b", Fallback: DefaultID},
			},
			want:    nil,
			wantErr: true,
		},
		{
			name: "Not found default",
			input: []*Locale{
				{ID: "a", Fallback: DefaultID},
				{ID: "b", Fallback: "d"},
				{ID: "c", Fallback: DefaultID},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
			},
			wantErr: true,
		},
		{
			name: "Simple",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: DefaultID},
				{ID: "b", Fallback: "d"},
				{ID: "c", Fallback: DefaultID},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
			},
			want: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: DefaultID},
				{ID: "c", Fallback: DefaultID},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
				{ID: "b", Fallback: "d"},
			},
			wantErr: false,
		},
		{
			name: "Simple #2",
			input: []*Locale{
				{ID: "a", Fallback: DefaultID},
				{ID: "b", Fallback: "d"},
				{ID: "c", Fallback: DefaultID},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
				{ID: DefaultID},
			},
			want: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: DefaultID},
				{ID: "c", Fallback: DefaultID},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
				{ID: "b", Fallback: "d"},
			},
			wantErr: false,
		},
		{
			name: "Simple #3",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a"},
				{ID: "b"},
			},
			want: []*Locale{
				{ID: DefaultID},
				{ID: "a"},
				{ID: "b"},
			},
			wantErr: false,
		},
		{
			name: "Simple #4",
			input: []*Locale{
				{ID: "a"},
				{ID: "b"},
				{ID: DefaultID},
			},
			want: []*Locale{
				{ID: DefaultID},
				{ID: "a"},
				{ID: "b"},
			},
			wantErr: false,
		},
		{
			name: "Simple #5",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: "b"},
				{ID: "b", Fallback: "c"},
				{ID: "c"},
			},
			want: []*Locale{
				{ID: DefaultID},
				{ID: "c"},
				{ID: "b", Fallback: "c"},
				{ID: "a", Fallback: "b"},
			},
			wantErr: false,
		},
		{
			name: "Simple #6",
			input: []*Locale{
				{ID: "a", Fallback: "b"},
				{ID: "b", Fallback: "c"},
				{ID: "c"},
			},
			want: []*Locale{
				{ID: "c"},
				{ID: "b", Fallback: "c"},
				{ID: "a", Fallback: "b"},
			},
			wantErr: false,
		},
		{
			name: "Cyclical dependency",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: "b"},
				{ID: "b", Fallback: "c"},
				{ID: "c", Fallback: "a"},
			},
			want:    nil,
			wantErr: true,
		},
		{
			name: "Cyclical dependency #2",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: DefaultID},
				{ID: "b", Fallback: "d"},
				{ID: "c", Fallback: "b"},
				{ID: "d", Fallback: "c"},
				{ID: "e", Fallback: "a"},
			},
			want:    nil,
			wantErr: true,
		},
		{
			name: "Cyclical dependency #3",
			input: []*Locale{
				{ID: DefaultID},
				{ID: "a", Fallback: "b"},
				{ID: "b", Fallback: "a"},
				{ID: "c", Fallback: "d"},
				{ID: "d", Fallback: "c"},
			},
			want:    nil,
			wantErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := FallbackSort(tt.input)
			if tt.wantErr {
				require.Error(t, err)
				return
			}
			require.NoError(t, err)
			assert.Equal(t, tt.want, got)
		})
	}
}
